from .queryResult import QueryResultBase
from typing import cast
from ..commonTyping import JsonType
from .repl import BasicRepl


class BasicComment(QueryResultBase):
    """A basic comment class."""

    id: str  #: Comment id
    repl: BasicRepl  #: Comment repl

    __slots__ = locals()["__annotations__"].keys()

    def __init__(self, json: JsonType) -> None:
        super().__init__(None, json)
        self.repl = BasicRepl(
            cast(JsonType, self.repl)["id"],
            cast(JsonType, self.repl)["url"],
            cast(JsonType, self.repl)["title"],
        )



from dataclasses import dataclass
from typing import Optional


@dataclass
class BasicRepl:
    """A basic repl dataclass."""

    id: str  #: The repl id
    url: str  #: The repl url
    title: Optional[str]  #: The repl title



import asyncio, aiohttp, aiolimiter
from .querys import querys
from .classes.user import User, SimpleUser
from .classes.queryResult import QueryResult, QueryResultBase
from .classes.notifications import makeNotification

# Typing
from types import CoroutineType
from .commonTyping import JsonType
from typing import Optional, Dict, Any, List


class RequestError(Exception):
    """Replit api request error."""

    pass


class ReplitClient:
    """The replit client object.
    This class is the centerpiece or the library,
    almost all of the functions are of this class.
    This should be the only object you need to import.

    :param sid: The replit account's secure ID, kind of like a token. Required for most actions but not all.
    """

    def __makeCookie(self) -> Dict[str, Optional[str] | int]:
        # Make the cookie needed to acess replit
        return {
            "connect.sid": self.sid,
            "replit_authed": 1,
        }
        # return f"connect.sid={self.sid}; replit_authed=1; replit_authed=1;"

    def __init__(self, sid: Optional[str] = None) -> None:  # Sid not always needed.
        self.sid = sid
        self.headers = {  # Headers needed to acess replit.
            "User-Agent": "replit",
            "Origin": "https://replit.com",
            "Referrer": "https://replit.com/",
            "Referrer-Policy": "strict-origin-when-cross-origin",
            "accept": "*/*",
            "accept-language": "en-US,en;q=0.8",
            "content-type": "application/json",
            "sec-fetch-dest": "empty",
            "sec-fetch-mode": "cors",
            "sec-fetch-site": "same-origin",
            "sec-gpc": "1",
            "x-requested-with": "XMLHttpRequest",
        }
        self.limiter = aiolimiter.AsyncLimiter(5)  # Current rate-limit is 5/s

    async def start(self) -> None:
        """Start the client, needs to be used first, before any other functions.

        .. warning::
           Never use with :meth:`autoreplit.ReplitClient.run`
        """
        cookies: Any = self.__makeCookie()
        self.client = aiohttp.ClientSession(cookies=cookies)

    async def close(self) -> None:
        """Close the client, needs to be run after everything is done.

        .. warning::
           Never use with :meth:`autoreplit.ReplitClient.run`
        """
        await self.client.close()

    def run(self, mainfunc: CoroutineType) -> None:
        """Run Coroutine mainfunc so that a replit client works withing.

        Uses a combination of :meth:`asyncio.run`, :meth:`autoreplit.ReplitClient.start` and
        :meth:`autoreplit.ReplitClient.close`
        """

        async def inner():
            await self.start()
            try:
                await mainfunc
            finally:
                await self.close()

        asyncio.run(inner())

    async def __gqlQuery(
        self, query: str, vars: JsonType, opname: Optional[str] = None
    ) -> JsonType:
        # Query replut graphql
        json: Dict[str, Any] = {}
        if opname != None:
            json["operationName"] = opname
        json = {"query": query, "variables": vars}
        session = self.client
        await self.limiter.acquire()  # Rate-limit
        async with session.post(
            "https://replit.com/graphql", json=json, headers=self.headers
        ) as response:
            if response.status != 200:
                raise RequestError(await response.text())  # Raise request errors
            json = await response.json()
            if "errors" in json:  # Raise other errors
                raise RequestError(json["errors"]["message"])
            return json

    async def rawQuery(
        self, queryname: str, query: str, vars: JsonType = {}
    ) -> QueryResult:
        """Use a raw gql query on the api. Returns a QueryResult object.

        :param queryname: The name of the query, for example ``UserByUsername``
        :param query: The actual graphql query
        :param vars: The variable params passed to the query
        """
        json = await self.__gqlQuery(query, vars, queryname)
        return QueryResult(queryname, json)

    async def getUserByName(self, name: str) -> User:
        """Get a user by name. Returns an user object."""
        query = querys["userByUsername"]
        result = await self.__gqlQuery(query, {"username": name}, "userByUsername")
        return User(result)

    async def getCurrentUser(self) -> SimpleUser:
        """Get the current user. Returns a ``SimpleUser``"""
        query = querys["currentUser"]
        result = await self.__gqlQuery(query, {}, "UpgradeModal")
        result = result["data"]["currentUser"]
        return SimpleUser(result["username"], result["id"])

    async def updatePresence(self) -> None:
        """Update your bot's presence, will set you to ``Online``"""
        query = querys["updatePresence"]
        await self.__gqlQuery(query, {}, "SitePresenceUpdate")

    async def getNotifications(
        self, count: int = 10, seen: bool = False
    ) -> List[QueryResultBase]:
        query = querys["notifications"]
        result = await self.__gqlQuery(
            query, {"count": count, "seen": seen}, "notifications"
        )
        return makeNotification(result)



from ..commonTyping import JsonType
from abc import ABCMeta
from typing import Optional


class ResultError(Exception):
    """Raised if result does not exist."""

    pass


class QueryResultBase(metaclass=ABCMeta):
    """The base object of :class:`.QueryResult`. It is also the base of all other result wrappers."""

    __slots__ = ()

    def __init__(self, opname: Optional[str], json: JsonType) -> None:
        if not opname is None:
            data = json["data"][opname]
        else:
            data = json
        if data is None:
            raise ResultError("Result does not exist.")
        key: str
        for key in self.__slots__:
            if key in data:
                self.__setattr__(key, data[key])
            else:
                self.__setattr__(key, None)

    def __str__(self) -> str:
        result = f"{self.__class__.__name__}("
        if len(self.__slots__) < 1:
            return result + ")"
        s: str
        for s in self.__slots__:
            result += f"{s}={getattr(self, s)}, "
        return result[:-2] + ")"


class QueryResult(QueryResultBase):
    """A un-slotted query result object for :meth:`autoreplit.ReplitClient.rawQuery`"""

    pass



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['autoreplit', 'autoreplit.classes']

package_data = \
{'': ['*'], 'autoreplit': ['gql/*']}

install_requires = \
['aiohttp>=3.8.3,<4.0.0', 'aiolimiter==1.0.0']

setup_kwargs = {
    'name': 'automate-replit',
    'version': '0.1.0',
    'description': 'Automate your replit account using this replit api wrapper!',
    'long_description': '# Automate Replit\n[![Documentation Status](https://readthedocs.org/projects/automate-replit/badge/?version=latest)](https://automate-replit.readthedocs.io/en/latest/?badge=latest)\n\n\nAutomate replit actions with this replit api wrapper!\n\n## Docs\nDocumentation for this library can be found [here](https://automate-replit.readthedocs.io/en/docs/)\n\n## Source\nSource can be found [here](https://github.com/thatrandomperson5/automate-replit)\n\n## Example\n```py\nfrom autoreplit import ReplitClient\n\nclient = ReplitClient()\n\nasync def getEthan():\n    ethan = await client.getUserByName("not-ethan")\n    print(f"Ethan\'s id: {ethan.id}")\n    print(f"Ethan\'s follower count: {ethan.followerCount}")\n    if ethan.isOnline:\n        print("Ethan is online!")\n    else:\n        print(f"Ethan was last seen {ethan.lastSeen}")\n    print(f"Ethan\'s roles: {ethan.roles}")\n    print(f"All of ethan: {ethan}")\n\nclient.run(getEthan())\n```\n**Note**: pass a sid to get better info',
    'author': 'thatrandomperson5',
    'author_email': 'None',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8.0,<4.0.0',
}


setup(**setup_kwargs)



from dataclasses import dataclass
from ..commonTyping import JsonType
from .queryResult import QueryResultBase
from typing import Dict, List, Optional


@dataclass
class Social:
    """Social contact object, like ``"GITHUB"``"""

    id: int
    url: str
    type: str


@dataclass
class Role:
    """A role object, provides role id, name, key and tagline."""

    id: str
    name: str
    key: str
    tagline: str


@dataclass
class SimpleUser:
    """A simple user object, provides username and id."""

    username: str
    id: str


class User(QueryResultBase):
    """The user object. Contains user data."""

    id: int  #: User's id
    username: str  #: User's username
    image: str  #: User's pfp
    url: str  #: Path to the users profile
    followerCount: int  #: How many followers the user has
    followCount: int  #: How many people the user is following
    isFollowedByCurrentUser: bool  # Self explanitory
    isFollowingCurrentUser: bool
    isBlockedByCurrentUser: bool
    isBlockingCurrentUser: bool
    fullName: str  #: The user's full name
    hasPrivacyRole: bool  #: Replit students under the age of 13 (probably)
    userSubscriptionType: Optional[
        str
    ]  #: User's plan, eg. ``"HACKER"``. ``None`` if the user has no plan

    #: A dictonary of roles, keys are the role key, eg. ``"MODERATOR"``.
    #: The values are of the :class:`.Role` class.
    roles: Dict[str, Role]

    publicRepls: List[str]  #: A list of the user's public repl's ids.
    presenceStatus: None  #: :meta private:

    #: ``True`` if the user is online
    #:
    #: .. note::
    #:   Always ``False`` if no SID is provided.
    isOnline: bool

    #: ``None`` if the user is online, Date-Time str if user is offline.
    #:
    #: .. note::
    #:   Always ``None`` if no SID is provided.
    lastSeen: Optional[str]

    #: A dictonary of social contacts, keys are the social type, eg. ``"github"``.
    #: The values are of the :class:`.Social` class.
    socials: Dict[str, Social]

    __slots__ = locals()["__annotations__"].keys()

    def __makePublicRepls(self):
        pr = self.publicRepls
        self.publicRepls = []
        for item in pr["items"]:
            self.publicRepls.append(item["id"])

    def __makePresenceStatus(self):
        for k, v in self.presenceStatus.items():
            self.__setattr__(k, v)
        self.presenceStatus = None

    def __makeRoles(self):
        oroles = self.roles  # Old roles
        self.roles = {}
        for item in oroles:
            self.roles[item["key"]] = Role(
                item["id"], item["name"], item["key"], item["tagline"]
            )

    def __makeSocials(self):
        osocials = self.socials
        self.socials = {}
        for item in osocials:
            self.socials[item["type"]] = Social(item["id"], item["url"], item["type"])

    def __init__(self, data: JsonType, pathName: str = "userByUsername") -> None:
        super().__init__(pathName, data)
        self.__makePublicRepls()
        self.__makePresenceStatus()
        self.__makeRoles()
        self.__makeSocials()



from pathlib import Path


def readFile(path: str | Path) -> str:
    path = Path(path)
    root = Path(__file__)
    path = root.parent / path
    with open(path, "r") as f:
        return f.read()


querys = {
    "userByUsername": readFile("gql/userByUsername.gql"),
    "updatePresence": readFile("gql/updatePresence.gql"),
    "currentUser": readFile("gql/currentUser.gql"),
    "notifications": readFile("gql/notifications.gql"),
}



from ..commonTyping import JsonType
from .user import SimpleUser
from .queryResult import QueryResultBase
from typing import List, Optional, cast
from warnings import warn
from .comments import BasicComment


class CommonNotif(QueryResultBase):
    """The most common stucture for notifications. Look at ``notifications.gql`` for more details."""

    id: str  #: Notification id
    timeCreated: str  #: Time the notification was created
    seen: bool  #: If the user saw the notification
    creator: SimpleUser  #: The notification's creator
    replComment: Optional[
        BasicComment
    ]  #: The linked comment, if the type is a comment notif
    type: str  #: Notif type
    url: Optional[str]  #: Notif url

    __slots__ = locals()["__annotations__"].keys()

    def __init__(self, json: JsonType) -> None:
        json["type"] = json["__typename"]
        del json["__typename"]
        super().__init__(None, json)
        self.creator = SimpleUser(
            cast(JsonType, self.creator)["username"], cast(JsonType, self.creator)["id"]
        )
        if not self.replComment is None:
            self.replComment = BasicComment(cast(JsonType, self.replComment))


class UnidentifiedNotif(QueryResultBase):
    """Unidentified notification. Will produce a warning, please featrue request support for the unidentified type."""

    def __init__(self, json: JsonType) -> None:
        json["type"] = json["__typename"]
        del json["__typename"]
        super().__init__(None, json)


def makeNotification(json: JsonType) -> List[QueryResultBase]:
    """Sort and type notif's."""
    data = json["data"]["notifications"]["items"]
    result: List[QueryResultBase] = []
    for item in data:
        match item["__typename"]:
            case "ReplCommentCreatedNotification" | "ReplCommentReplyCreatedNotification" | "ReplCommentMentionNotification" | "NewFollowerNotification":
                result.append(CommonNotif(item))
            case _:
                warn(f"Unwrapped and Unidentified notif {item['__typename']}")
                result.append(UnidentifiedNotif(item))
    return result
